/*
 * Generated by asn1c-0.9.28 (http://lionet.info/asn1c)
 * From ASN.1 module "RANAP-IEs"
 * 	found in "../../asn1/ranap/RANAP-IEs.asn"
 */

#ifndef	_RANAP_InformationRequestType_H_
#define	_RANAP_InformationRequestType_H_


#include <asn_application.h>

/* Including external dependencies */
#include <osmocom/ranap/RANAP_MBMSIPMulticastAddressandAPNRequest.h>
#include <osmocom/ranap/RANAP_PermanentNAS-UE-ID.h>
#include <constr_CHOICE.h>

#ifdef __cplusplus
extern "C" {
#endif

/* Dependencies */
typedef enum RANAP_InformationRequestType_PR {
	RANAP_InformationRequestType_PR_NOTHING,	/* No components present */
	RANAP_InformationRequestType_PR_mBMSIPMulticastAddressandAPNRequest,
	RANAP_InformationRequestType_PR_permanentNAS_UE_ID,
	/* Extensions may appear below */
	
} RANAP_InformationRequestType_PR;

/* RANAP_InformationRequestType */
typedef struct RANAP_InformationRequestType {
	RANAP_InformationRequestType_PR present;
	union RANAP_InformationRequestType_u {
		RANAP_MBMSIPMulticastAddressandAPNRequest_t	 mBMSIPMulticastAddressandAPNRequest;
		RANAP_PermanentNAS_UE_ID_t	 permanentNAS_UE_ID;
		/*
		 * This type is extensible,
		 * possible extensions are below.
		 */
	} choice;
	
	/* Context for parsing across buffer boundaries */
	asn_struct_ctx_t _asn_ctx;
} RANAP_InformationRequestType_t;

/* Implementation */
extern asn_TYPE_descriptor_t asn_DEF_RANAP_InformationRequestType;

#ifdef __cplusplus
}
#endif

#endif	/* _RANAP_InformationRequestType_H_ */
#include <asn_internal.h>
